from __future__ import print_function
from gevent import monkey
monkey.patch_all()

import os
import unittest
import multiprocessing

import gevent

hub = gevent.get_hub()
pid = os.getpid()
newpid = None


def on_fork():
    global newpid
    newpid = os.getpid()

fork_watcher = hub.loop.fork(ref=False)
fork_watcher.start(on_fork)


def in_child(q):
    # libev only calls fork callbacks at the beginning of
    # the loop; we use callbacks extensively so it takes *two*
    # calls to sleep (with a timer) to actually get wrapped
    # around to the beginning of the loop.
    gevent.sleep(0.001)
    gevent.sleep(0.001)
    q.put(newpid)


class Test(unittest.TestCase):

    def test(self):
        self.assertEqual(hub.threadpool.size, 0)
        # Use a thread to make us multi-threaded
        hub.threadpool.apply(lambda: None)
        self.assertEqual(hub.threadpool.size, 1)

        # Not all platforms use fork by default, so we want to force it,
        # where possible. The test is still useful even if we can't
        # fork though.
        try:
            fork_ctx = multiprocessing.get_context('fork')
        except (AttributeError, ValueError):
            # ValueError if fork isn't supported.
            # AttributeError on Python 2, which doesn't have get_context
            fork_ctx = multiprocessing

        # If the Queue is global, q.get() hangs on Windows; must pass as
        # an argument.
        q = fork_ctx.Queue()
        p = fork_ctx.Process(target=in_child, args=(q,))
        p.start()
        p.join()
        p_val = q.get()

        self.assertIsNone(
            newpid,
            "The fork watcher ran in the parent for some reason."
        )
        self.assertIsNotNone(
            p_val,
            "The child process returned nothing, meaning the fork watcher didn't run in the child."
        )
        self.assertNotEqual(p_val, pid)
        assert p_val != pid

if __name__ == '__main__':
    # Must call for Windows to fork properly; the fork can't be in the top-level
    multiprocessing.freeze_support()

    # fork watchers weren't firing in multi-threading processes.
    # This test is designed to prove that they are.
    # However, it fails on Windows: The fork watcher never runs!
    # This makes perfect sense: on Windows, our patches to os.fork()
    # that call gevent.hub.reinit() don't get used; os.fork doesn't
    # exist and multiprocessing.Process uses the windows-specific _subprocess.CreateProcess()
    # to create a whole new process that has no relation to the current process;
    # that process then calls multiprocessing.forking.main() to do its work.
    # Since no state is shared, a fork watcher cannot exist in that process.
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test for issue #1526:
- dnspython is imported first;
- no monkey-patching is done.
"""
from __future__ import print_function
from __future__ import absolute_import

import dns # pylint:disable=import-error
assert dns
import gevent.socket as socket # pylint:disable=consider-using-from-import
socket.getfqdn() # create the resolver

from gevent.resolver.dnspython import dns as gdns
import dns.rdtypes # pylint:disable=import-error

assert dns is not gdns, (dns, gdns)
assert dns.rdtypes is not gdns.rdtypes
import sys
print(sorted(sys.modules))




try:
    # Do this before the patch to be sure we clean
    # things up properly if the order is wrong.
    import selectors
except ImportError:
    import selectors2 as selectors

from gevent.monkey import patch_all
import gevent.testing as greentest

patch_all()

from gevent.selectors import DefaultSelector
from gevent.selectors import GeventSelector
from gevent.tests.test__selectors import SelectorTestMixin

class TestSelectors(SelectorTestMixin, greentest.TestCase):

    @greentest.skipOnWindows(
        "SelectSelector._select is a normal function on Windows"
    )
    def test_selectors_select_is_patched(self):
        # https://github.com/gevent/gevent/issues/835
        _select = selectors.SelectSelector._select
        self.assertIn('_gevent_monkey', dir(_select))

    def test_default(self):
        # Depending on the order of imports, gevent.select.poll may be defined but
        # selectors.PollSelector may not be defined.
        # https://github.com/gevent/gevent/issues/1466
        self.assertIs(DefaultSelector, GeventSelector)
        self.assertIs(selectors.DefaultSelector, GeventSelector)

    def test_import_selectors(self):
        # selectors can always be imported once monkey-patched. On Python 2,
        # this is an alias for gevent.selectors.
        __import__('selectors')

    def _make_test(name, kind): # pylint:disable=no-self-argument
        if kind is None:
            def m(self):
                self.skipTest(name + ' is not defined')
        else:
            def m(self, k=kind):
                with k() as sel:
                    self._check_selector(sel)
        m.__name__ = 'test_selector_' + name
        return m

    SelKind = SelKindName = None
    for SelKindName in (
            # The subclass hierarchy changes between versions, and is
            # complex (e.g, BaseSelector <- BaseSelectorImpl <-
            # _PollLikSelector <- PollSelector) so its easier to check against
            # names.
            'KqueueSelector',
            'EpollSelector',
            'DevpollSelector',
            'PollSelector',
            'SelectSelector',
            GeventSelector,
    ):
        if not isinstance(SelKindName, type):
            SelKind = getattr(selectors, SelKindName, None)
        else:
            SelKind = SelKindName
            SelKindName = SelKind.__name__
        m = _make_test(SelKindName, SelKind) # pylint:disable=too-many-function-args
        locals()[m.__name__] = m

    del SelKind
    del SelKindName
    del _make_test



if __name__ == '__main__':
    greentest.main()



from __future__ import absolute_import
from __future__ import print_function

from zope.interface import implementer

from gevent._interfaces import ICallback

__all__ = [
    'callback',
]


@implementer(ICallback)
class callback(object):

    __slots__ = ('callback', 'args')

    def __init__(self, cb, args):
        self.callback = cb
        self.args = args

    def stop(self):
        self.callback = None
        self.args = None

    close = stop

    # Note that __nonzero__ and pending are different
    # bool() is used in contexts where we need to know whether to schedule another callback,
    # so it's true if it's pending or currently running
    # 'pending' has the same meaning as libev watchers: it is cleared before actually
    # running the callback

    def __bool__(self):
        # it's nonzero if it's pending or currently executing
        # NOTE: This depends on loop._run_callbacks setting the args property
        # to None.
        return self.args is not None

    @property
    def pending(self):
        return self.callback is not None

    def _format(self):
        return ''

    def __repr__(self):
        result = "<%s at 0x%x" % (self.__class__.__name__, id(self))
        if self.pending:
            result += " pending"
        if self.callback is not None:
            result += " callback=%r" % (self.callback, )
        if self.args is not None:
            result += " args=%r" % (self.args, )
        if self.callback is None and self.args is None:
            result += " stopped"
        return result + ">"
